@@ -95,6 +95,12 @@ class Blob:
         -------
 
         """
+
+        if not set([account]).issubset(set(ACCOUNT.__args__)):
+            logger.error('account parameter must be set to one of the allowed values defined by the '
+                         'ACCOUNT literal: %s', ACCOUNT.__args__)
+            return False
+
         con_string = _get_connection_string(api_inputs, account)
 
         if prefix is None:
@@ -126,6 +132,11 @@ class Blob:
         -------
 
         """
+        if not set([account]).issubset(set(ACCOUNT.__args__)):
+            logger.error('account parameter must be set to one of the allowed values defined by the '
+                         'ACCOUNT literal: %s', ACCOUNT.__args__)
+            return False
+
         con_string = _get_connection_string(api_inputs, account)
         blob_service_client = BlobServiceClient.from_connection_string(con_string)
         container_client = blob_service_client.get_container_client(container)
@@ -143,8 +154,8 @@ class Blob:
         ----------
         api_inputs : ApiInputs
             Object returned by initialize() function.
-        account: ACCOUNT
-            Blob account data will be uploaded to.
+        account: ACCOUNT, optional
+            Blob account data will be uploaded to (Default value = 'SwitchStorage').
         data_frame : pandas.DataFrame
             Dataframe containing the data to be uploaded to blob.
         name : str
@@ -162,6 +173,11 @@ class Blob:
         """
         chunk_size = 1000000
 
+        if not set([account]).issubset(set(ACCOUNT.__args__)):
+            logger.error('account parameter must be set to one of the allowed values defined by the '
+                         'ACCOUNT literal: %s', ACCOUNT.__args__)
+            return False
+
         con_string = _get_connection_string(api_inputs, account)
         blob_service_client = BlobServiceClient.from_connection_string(con_string)
         container_client = blob_service_client.get_container_client("data-ingestion-adx")
@@ -198,6 +214,11 @@ def _get_connection_string(api_inputs: ApiInputs, account: ACCOUNT = 'SwitchStor
 
     """
 
+    if not set([account]).issubset(set(ACCOUNT.__args__)):
+        logger.error('account parameter must be set to one of the allowed values defined by the '
+                     'ACCOUNT literal: %s', ACCOUNT.__args__)
+        return False
+
     headers = {
         'x-functions-key': api_inputs.api_key,
         'Content-Type': 'application/json; charset=utf-8',
