@@ -35,8 +35,9 @@ import pandera
 import logging
 import requests
 import sys
+import json
 from typing import Optional, List, Union
-from .._utils._constants import api_prefix, ERROR_TYPE
+from .._utils._constants import api_prefix, ERROR_TYPE, PROCESS_STATUS
 from .._utils._utils import ApiInputs
 
 logger = logging.getLogger(__name__)
@@ -100,7 +101,8 @@ def invalid_file_format(api_inputs: ApiInputs, schema: pandera.DataFrameSchema,
         return response_data_frame
 
 
-def post_errors(api_inputs: ApiInputs, errors_df: pandas.DataFrame, error_type: ERROR_TYPE):
+def post_errors(api_inputs: ApiInputs, errors_df: pandas.DataFrame, error_type: ERROR_TYPE,
+                process_status: PROCESS_STATUS = 'ActionRequired'):
     """Post errors to the Data Feed Dashboard
 
     Post dataframe containing the errors of type ``error_type`` to the Data Feed Dashboard in the Switch Platform.
@@ -113,7 +115,10 @@ def post_errors(api_inputs: ApiInputs, errors_df: pandas.DataFrame, error_type:
         The dataframe containing the rows with errors.
     error_type : ERROR_TYPE
         The type of error being posted to Data Feed Dashboard.
-
+    process_status: PROCESS_STATUS, optional
+        Set the status of the process to one of the allowed values specified by the PROCESS_STATUS literal
+        (Default value = 'ActionRequired').
+        
     Returns
     -------
     response_boolean: boolean
@@ -124,15 +129,29 @@ def post_errors(api_inputs: ApiInputs, errors_df: pandas.DataFrame, error_type:
         logger.error("You must call initialize() before using API.")
         return False
 
+    if (api_inputs.data_feed_id == '00000000-0000-0000-0000-000000000000' or api_inputs.data_feed_file_status_id == '00000000-0000-0000-0000-000000000000'):
+        logger.error("Post Errors can only be called in Production.")
+        return False
+
+    if not set([error_type]).issubset(set(ERROR_TYPE.__args__)):
+        logger.error('error_type parameter must be set to one of the allowed values defined by the '
+                     'ERROR_TYPE literal: %s', ERROR_TYPE.__args__)
+        return False
+
+    if not set([process_status]).issubset(set(PROCESS_STATUS.__args__)):
+        logger.error('process_status parameter must be set to one of the allowed values defined by the '
+                     'PROCESS_STATUS literal: %s', PROCESS_STATUS.__args__)
+        return False
+
     headers = {
         'x-functions-key': api_inputs.api_key,
         'Content-Type': 'application/json; charset=utf-8',
         'user-key': api_inputs.user_id
     }
-    url = api_prefix + api_inputs.datacentre + "/" + api_inputs.api_project_id + "/DataFeedErrors/" + error_type + \
-        "/" + str(api_inputs.data_feed_id) + "/" + str(api_inputs.data_feed_file_status_id)
+    url = api_prefix + api_inputs.datacentre + "/" + api_inputs.api_project_id + "/DataFeedErrors/" + str(error_type) \
+        + "/" + str(api_inputs.data_feed_id) + "/" + str(api_inputs.data_feed_file_status_id)
 
-    response = requests.post(url, data=errors_df.to_json(orient='records'), headers=headers)
+    response = requests.post(url, data=json.dumps(json.loads(errors_df.to_json(orient='table'))['data']), headers=headers)
 
     response_status = '{} {}'.format(response.status_code, response.reason)
     if response.status_code != 200:
@@ -143,6 +162,8 @@ def post_errors(api_inputs: ApiInputs, errors_df: pandas.DataFrame, error_type:
         logger.error('No data returned for this API call. %s', response.request.__dict__)
         return response_status, False
 
+    logger.error(f'Directive:ProcessStatus={str(process_status)}')
+
     return response.text
 
 
@@ -226,153 +247,3 @@ def validate_datetime(df: pandas.DataFrame, datetime_col: Union[str, List[str]],
         logger.info('Row count with invalid datetime values: %s', df_invalid_datetime.shape[0])
         logger.info('Row count with valid datetime values: %s', df.shape[0])
         return df_invalid_datetime, df
-
-# def missing_devices(api_inputs: ApiInputs, missing_devices_df: pandas.DataFrame):
-#     """Send devices unable to be identified in platform to data feed dashboard.
-#
-#     Any records in the data file that cannot be aligned to a device within the Switch Automation platform.
-#
-#     Parameters
-#     ----------
-#     api_inputs : ApiInputs
-#         Object returned by call to initialize()
-#     missing_devices_df : pandas.DataFrame
-#         The dataframe containing the unique instances of devices that do not match any existing in the Switch
-#         platform.
-#
-#     Returns
-#     -------
-#     response_data_frame : pandas.DataFrame
-#         Data frame containing the response from API after posting any errors to data feed dashboard.
-#
-#     """
-#     if api_inputs.datacentre == '' or api_inputs.api_key == '':
-#         logger.error("You must call initialize() before using API.")
-#         return pandas.DataFrame()
-#
-#     headers = {
-#         'x-functions-key': api_inputs.api_key,
-#         'Content-Type': 'application/json; charset=utf-8',
-#         'user-key': api_inputs.user_id
-#     }
-#
-#     url = api_prefix + api_inputs.datacentre + "/" + api_inputs.api_project_id + "/DataFeedErrors/MissingDevices"
-#     response = requests.post(url, data=missing_devices_df.to_json(orient='records'), headers=headers)
-#     response_status = '{} {}'.format(response.status_code, response.reason)
-#     if response.status_code != 200:
-#         logger.error("API Call was not successful. Response Status: %s. Reason: %s.", response.status_code,
-#                      response.reason)
-#         return response_status, pandas.DataFrame()
-#     elif len(response.text) == 0:
-#         logger.error('No data returned for this API call. %s', response.request.__dict__)
-#         return response_status, pandas.DataFrame()
-#
-#     response_data_frame = pandas.read_json(response.text)
-#
-#     return response_data_frame
-
-
-# def invalid_datetime(api_inputs: ApiInputs, df: pandas.DataFrame, datetime_col: List[str],
-#                      dt_fmt: Optional[str] = None): # -> tuple[pandas.DataFrame, pandas.DataFrame]:
-#     """Check for datetime errors.
-#
-#     Returns tuple in the form ``(df_invalid_datetime, df)`` where:
-#         - ``df_invalid_datetime`` is a dataframe containing the extracted rows of the input `df` with invalid
-#         datetime values.
-#
-#         - ``df`` is the original dataframe input after dropping any rows with datetime errors
-#
-#     Parameters
-#     ----------
-#     api_inputs: ApiInputs
-#         The object returned by call to initialize()
-#     df : pandas.DataFrame
-#         The dataframe that contains the columns to be validated.
-#     datetime_col: List[str]
-#         List of column names
-#     dt_fmt : Optional[str]
-#         The expected format of the datetime columns to be coerced. The strftime to parse time, eg "%d/%m/%Y", note
-#         that "%f" will parse all the way up to nanoseconds. See strftime documentation for more information on
-#         choices: https://docs.python.org/3/library/datetime.html#strftime-and-strptime-behavior
-#
-#     Returns
-#     -------
-#     df_invalid_datetime, df : tuple[pandas.DataFrame, pandas.DataFrame]
-#
-#         *df_invalid_datetime* : Dataframe containing the extracted rows of the input df with invalid datetime values.
-#
-#         df: The original dataframe input after dropping any rows with datetime errors.
-#     df : pandas.DataFrame
-#         The original dataframe input after dropping any rows with datetime errors
-#
-#     """
-#
-#     if api_inputs.datacentre == '' or api_inputs.api_key == '':
-#         logger.error("You must call initialize() before using API.")
-#         return pandas.DataFrame()
-#
-#     headers = {
-#         'x-functions-key': api_inputs.api_key,
-#         'Content-Type': 'application/json; charset=utf-8',
-#         'user-key': api_inputs.user_id
-#     }
-#     url = api_prefix + api_inputs.datacentre + "/" + api_inputs.api_project_id + "/DataFeedErrors/DateTime"
-#
-#     if type(datetime_col) == str:
-#         datetime_col = [datetime_col]
-#     elif type(datetime_col) == list:
-#         datetime_col = datetime_col
-#     else:
-#         return 'datetime_col: Invalid format. datetime_col must be a string or list of strings.'
-#
-#     val_dt_cols = []
-#     for i in datetime_col:
-#         val_dt_cols.append(i + '_dt')
-#     lst = [None] * len(datetime_col)
-#
-#     if dt_fmt is None:
-#         for i in range(len(datetime_col)):
-#             df[val_dt_cols[i]] = pandas.to_datetime((df[datetime_col[i]]), errors='coerce')
-#             lst[i] = df[df[val_dt_cols[i]].isnull()]
-#             df = df[df[val_dt_cols[i]].notnull()]
-#             lst[i] = lst[i].drop(val_dt_cols[i], axis=1)
-#             df = df.drop(val_dt_cols[i], axis=1)
-#         df_invalid_datetime = pandas.concat(lst, axis=0)
-#         df[datetime_col] = df[datetime_col].apply(lambda x: pandas.to_datetime(x))
-#
-#         response = requests.post(url, data=df_invalid_datetime.to_json(orient='records'), headers=headers)
-#         response_status = '{} {}'.format(response.status_code, response.reason)
-#         if response.status_code != 200:
-#             logger.error("API Call was not successful. Response Status: %s. Reason: %s.", response.status_code,
-#                          response.reason)
-#             return response_status, pandas.DataFrame()
-#         elif len(response.text) == 0:
-#             logger.error('No data returned for this API call. %s', response.request.__dict__)
-#             return response_status, pandas.DataFrame()
-#
-#         response_data_frame = pandas.read_json(response.text)
-#
-#         return df_invalid_datetime, df
-#     else:
-#         for i in range(len(datetime_col)):
-#             df[val_dt_cols[i]] = pandas.to_datetime((df[datetime_col[i]]), errors='coerce', format=dt_fmt)
-#             lst[i] = df[df[val_dt_cols[i]].isnull()]
-#             df = df[df[val_dt_cols[i]].notnull()]
-#             lst[i] = lst[i].drop(val_dt_cols[i], axis=1)
-#             df = df.drop(val_dt_cols[i], axis=1)
-#         df_invalid_datetime = pandas.concat(lst, axis=0)
-#         df[datetime_col] = df[datetime_col].apply(lambda x: pandas.to_datetime(x, format=dt_fmt))
-#
-#         response = requests.post(url, data=df_invalid_datetime.to_json(orient='records'), headers=headers)
-#         response_status = '{} {}'.format(response.status_code, response.reason)
-#         if response.status_code != 200:
-#             logger.error("API Call was not successful. Response Status: %s. Reason: %s.", response.status_code,
-#                          response.reason)
-#             return response_status, pandas.DataFrame()
-#         elif len(response.text) == 0:
-#             logger.error('No data returned for this API call. %s', response.request.__dict__)
-#             return response_status, pandas.DataFrame()
-#
-#         response_data_frame = pandas.read_json(response.text)
-#
-#         return df_invalid_datetime, df
