@@ -326,8 +326,8 @@ def get_data(query_text, api_inputs: ApiInputs, query_language: QUERY_LANGUAGE =
         SQL statement used to retrieve data.
     api_inputs : ApiInputs
         Object returned by initialize() function.
-    query_language : QUERY_LANGUAGE
-        The query language the query_text is written in.
+    query_language : QUERY_LANGUAGE, optional
+        The query language the query_text is written in (Default value = 'sql').
 
     Returns
     -------
@@ -348,6 +348,11 @@ def get_data(query_text, api_inputs: ApiInputs, query_language: QUERY_LANGUAGE =
         logger.error("You must call initialize() before using API.")
         return pandas.DataFrame()
 
+    if not set([query_language]).issubset(set(QUERY_LANGUAGE.__args__)):
+        logger.error('query_language parameter must be set to one of the allowed values defined by the '
+                     'QUERY_LANGUAGE literal: %s', QUERY_LANGUAGE.__args__)
+        return False
+
     url = api_prefix + api_inputs.datacentre + "/" + api_inputs.db_name + "/" + api_inputs.api_project_id + \
         "/Integration/GetData"
     response = requests.post(url, json=payload, headers=headers)
