@@ -65,8 +65,10 @@ from typing import List
 from abc import ABC, abstractmethod
 from azure.servicebus import ServiceBusClient, ServiceBusReceiveMode  # , ServiceBusMessage
 from .._utils._utils import _is_valid_regex, ApiInputs, generate_password, _column_name_cap, DataFeedFileProcessOutput
-from .._utils._constants import api_prefix, argus_prefix, EXPECTED_DELIVERY, DEPLOY_TYPE, MAPPING_ENTITIES, QUEUE_NAME
+from .._utils._constants import (api_prefix, argus_prefix, EXPECTED_DELIVERY, DEPLOY_TYPE, MAPPING_ENTITIES,
+                                 QUEUE_NAME, ERROR_TYPE)
 from .._utils._platform import _get_connection_string
+from io import StringIO
 
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.DEBUG)
@@ -81,8 +83,7 @@ consoleHandler.setFormatter(formatter)
 
 # Base Definitions in Switch package
 class Task(ABC):
-    """
-    An Abstract Base Class called Task.
+    """An Abstract Base Class called Task.
 
     Attributes
     ----------
@@ -91,7 +92,7 @@ class Task(ABC):
         A new unique identifier can be created using uuid.uuid4()
     description : str
         Brief description of the task
-    mapping_entities : MAPPING_ENTITIES
+    mapping_entities : List[MAPPING_ENTITIES]
         The type of entities being mapped. An example is:
 
         ``['Installations', 'Devices', 'Sensors']``
@@ -116,7 +117,7 @@ class Task(ABC):
 
     @property
     @abstractmethod
-    def mapping_entities(self) -> MAPPING_ENTITIES:
+    def mapping_entities(self) -> List[MAPPING_ENTITIES]:
         """The type of entities being mapped."""
         pass
 
@@ -384,7 +385,7 @@ class Automation:
             'user-key': api_inputs.user_id
         }
 
-        url = argus_prefix + "ReserveInstance/" + data_feed_id + "/" + str(minutes_to_reserve)
+        url = argus_prefix + "ReserveInstance/" + str(data_feed_id) + "/" + str(minutes_to_reserve)
 
         response = requests.request("GET", url, timeout=20, headers=headers)
         response_status = '{} {}'.format(response.status_code, response.reason)
@@ -423,6 +424,7 @@ class Automation:
                 "Driver must be an implementation of the Abstract Base Class (Task).")
             return False
 
+
         base_class = ''
         if issubclass(type(task), IntegrationTask):
             base_class = 'IntegrationTask'
@@ -433,14 +435,22 @@ class Automation:
 
         if base_class == '':
             logger.error(
-                'Task must be an implementation of the Task Subclasses of AnalyticsTask, IntegrationTask or '
+                'Task must be an implementation of one of the Task sub-classes: AnalyticsTask, IntegrationTask or '
                 'LogicModule.')
-            return {}, pandas.DataFrame()
+            return pandas.DataFrame()
+
+        if type(task.mapping_entities) != list:
+            logger.error('The mapping_entities parameter must have type = list. ')
+            return pandas.DataFrame()
+        elif not set(task.mapping_entities).issubset(set(MAPPING_ENTITIES.__args__)):
+            logger.error('mapping_entities property must be a list containing one of the allowed values defined by the '
+                         'MAPPING_ENTITIES literal: %s', MAPPING_ENTITIES.__args__)
+            return pandas.DataFrame()
 
         logger.info("Registering %s (%s) to the Switch Driver Library: ", str(type(task).__name__), str(task.id))
 
         json_payload_verify = {
-            "driverId": task.id,
+            "driverId": str(task.id),
             "name": type(task).__name__
         }
 
@@ -456,7 +466,7 @@ class Automation:
             return response_status, pandas.DataFrame()
 
         json_payload = {
-            "driverId": task.id,
+            "driverId": str(task.id),
             "name": type(task).__name__,
             "specification": task.description,
             "mappingEntities": task.mapping_entities,
@@ -483,7 +493,7 @@ class Automation:
 
     @staticmethod
     def list_tasks(api_inputs: ApiInputs, search_name_pattern: str = '*'):
-        """Get a list of the deployed tasks.
+        """Get a list of the registered tasks.
 
         Parameters
         ----------
@@ -496,6 +506,7 @@ class Automation:
         Returns
         -------
         pandas.DataFrame
+            Dataframe containing the registered tasks.
 
         """
         if api_inputs.datacentre == '' or api_inputs.api_key == '':
@@ -526,10 +537,11 @@ class Automation:
 
         return df
 
+
     @staticmethod
-    def deploy_as_email_data_feed(task, api_inputs: ApiInputs, email_address_domain='SwitchAutomation.com',
-                                  email_subject_regex='', queue_name: QUEUE_NAME = "task",
-                                  expected_delivery: EXPECTED_DELIVERY = "Daily"):
+    def deploy_as_email_data_feed(task, api_inputs: ApiInputs, expected_delivery: EXPECTED_DELIVERY,
+                                  email_subject_regex: str, email_address_domain='switchautomation.com',
+                                  queue_name: QUEUE_NAME = 'task'):
         """Deploy task as an email data feed.
 
         Deploys the created `task` as an email data feed. This allows the driver to ingest data sent via email. The
@@ -539,18 +551,18 @@ class Automation:
         Parameters
         ----------
         task : Task
-            The custom driver class created from the Abstract Base Class `Driver`
+            The custom driver class created from the Abstract Base Class `Task`
         api_inputs : ApiInputs
             Object returned by initialize() function.
-        email_address_domain : str, default='switchautomation.com'
-            The email domain, without the @ symbol, of the sender (Default value = 'SwitchAutomation.com').
+        expected_delivery : EXPECTED_DELIVERY
+            The expected delivery frequency.
         email_subject_regex : str
             Regex expression used to parse the email subject line to determine which driver the received file will
             be processed by (Default value = '').
-        queue_name : QUEUE_NAME
-            The name of queue.
-        expected_delivery : EXPECTED_DELIVERY
-            The expected delivery frequency (Default value = "Daily").
+        email_address_domain : str, default='switchautomation.com'
+            The email domain, without the @ symbol, of the sender (Default value = 'SwitchAutomation.com').
+        queue_name : QUEUE_NAME, optional
+            The name of queue (Default value = 'task').
 
         Returns
         -------
@@ -577,6 +589,16 @@ class Automation:
             logger.error("Do not include the @ in the email_address_domain parameter. ")
             return pandas.DataFrame()
 
+        if not set([expected_delivery]).issubset(set(EXPECTED_DELIVERY.__args__)):
+            logger.error('expected_delivery parameter must be set to one of the allowed values defined by the '
+                         'EXPECTED_DELIVERY literal: %s', EXPECTED_DELIVERY.__args__)
+            return pandas.DataFrame()
+
+        if not set([queue_name]).issubset(set(QUEUE_NAME.__args__)):
+            logger.error('queue_name parameter must be set to one of the allowed values defined by the '
+                         'QUEUE_NAME literal: %s', QUEUE_NAME.__args__)
+            return pandas.DataFrame()
+
         headers = {
             'x-functions-key': api_inputs.api_key,
             'Content-Type': 'application/json; charset=utf-8',
@@ -585,7 +607,7 @@ class Automation:
 
         inbox_container = "data-exchange"
         payload = {
-            "driverId": task.id,
+            "driverId": str(task.id),
             "name": 'Feed Settings for ' + type(task).__name__,
             "feedType": ",".join(task.mapping_entities),
             "expectedDelivery": expected_delivery,
@@ -610,13 +632,14 @@ class Automation:
             logger.error('No data returned for this API call. %s', response.request.__dict__)
             return response_status, pandas.DataFrame()
 
-        df = pandas.read_json(response.text, typ='Series')
+        df = pandas.read_json(response.text, typ='Series').to_frame().T
+        df.columns = _column_name_cap(df.columns)
 
         return df
 
     @staticmethod
     def deploy_as_ftp_data_feed(task, api_inputs: ApiInputs, ftp_user_name, ftp_password,
-                                queue_name: QUEUE_NAME = "task", expected_delivery: EXPECTED_DELIVERY = "15min"):
+                                expected_delivery: EXPECTED_DELIVERY, queue_name: QUEUE_NAME = 'task'):
         """Deploy the custom driver as an FTP data feed
 
         Deploys the custom driver to receive data via an FTP data feed. Sets the `ftp_user_name` & `ftp_password` and
@@ -625,7 +648,7 @@ class Automation:
         Parameters
         ----------
         task : Task
-            The custom driver class created from the Abstract Base Class 'Driver'
+            The custom driver class created from the Abstract Base Class 'Task'
         api_inputs : ApiInputs
             Object returned by the initialize() function.
         ftp_user_name : str
@@ -633,10 +656,10 @@ class Automation:
         ftp_password : str
             The password to be used by the ftp service for the given `ftp_user_name` to authenticate delivery of the
             data feed.
+        expected_delivery : EXPECTED_DELIVERY
+            The expected delivery frequency of the data.
         queue_name : QUEUE_NAME, default = 'task'
-            The queue name.
-        expected_delivery : EXPECTED_DELIVERY, default = '15min'
-            The expected delivery frequency of the data (Default value = "15Minutes").
+            The queue name (Default value = 'task').
 
         Returns
         -------
@@ -649,6 +672,16 @@ class Automation:
             logger.error("You must call initialize() before using API.")
             return pandas.DataFrame()
 
+        if not set([expected_delivery]).issubset(set(EXPECTED_DELIVERY.__args__)):
+            logger.error('expected_delivery parameter must be set to one of the allowed values defined by the '
+                         'EXPECTED_DELIVERY literal: %s', EXPECTED_DELIVERY.__args__)
+            return pandas.DataFrame()
+
+        if not set([queue_name]).issubset(set(QUEUE_NAME.__args__)):
+            logger.error('queue_name parameter must be set to one of the allowed values defined by the '
+                         'QUEUE_NAME literal: %s', QUEUE_NAME.__args__)
+            return pandas.DataFrame()
+
         logger.info('Deploy %s (%s) as a data feed for ApiProjectID: %s', type(task).__name__, str(task.id),
                     api_inputs.api_project_id)
 
@@ -660,7 +693,7 @@ class Automation:
 
         inbox_container = "data-exchange"
         payload = {
-            "driverId": task.id,
+            "driverId": str(task.id),
             "name": 'Feed Settings for ' + type(task).__name__,
             "feedType": ",".join(task.mapping_entities),
             "expectedDelivery": expected_delivery,
@@ -690,8 +723,8 @@ class Automation:
         return df
 
     @staticmethod
-    def deploy_as_upload_data_feed(task, api_inputs: ApiInputs, queue_name: QUEUE_NAME = "task",
-                                   expected_delivery: EXPECTED_DELIVERY = "15min"):
+    def deploy_as_upload_data_feed(task, api_inputs: ApiInputs, expected_delivery: EXPECTED_DELIVERY,
+                                   queue_name: QUEUE_NAME = 'task'):
         """Deploy the custom driver as a REST API end point Datafeed.
 
         Parameters
@@ -700,10 +733,11 @@ class Automation:
             The custom driver class created from the Abstract Base Class 'Driver'
         api_inputs : ApiInputs
             Object returned by the initialize() function.
-        queue_name : QUEUE_NAME, default = 'task'
-            The queue name.
-        expected_delivery : EXPECTED_DELIVERY, default = '15min'
-            The expected delivery frequency of the data (Default value = "15Minutes").
+        expected_delivery : EXPECTED_DELIVERY
+            The expected delivery frequency of the data.
+        queue_name : QUEUE_NAME, optional
+            The queue name (Default value = 'task').
+
 
         Returns
         -------
@@ -716,6 +750,16 @@ class Automation:
             logger.error("You must call initialize() before using API.")
             return pandas.DataFrame()
 
+        if not set([expected_delivery]).issubset(set(EXPECTED_DELIVERY.__args__)):
+            logger.error('expected_delivery parameter must be set to one of the allowed values defined by the '
+                         'EXPECTED_DELIVERY literal: %s', EXPECTED_DELIVERY.__args__)
+            return pandas.DataFrame()
+
+        if not set([queue_name]).issubset(set(QUEUE_NAME.__args__)):
+            logger.error('queue_name parameter must be set to one of the allowed values defined by the '
+                         'QUEUE_NAME literal: %s', QUEUE_NAME.__args__)
+            return pandas.DataFrame()
+
         logger.info('Deploy %s (%s) as a data feed for ApiProjectID: %s', type(task).__name__, str(task.id),
                     api_inputs.api_project_id)
 
@@ -727,7 +771,7 @@ class Automation:
 
         # inbox_container = "data-exchange"
         payload = {
-            "driverId": task.id,
+            "driverId": str(task.id),
             "name": 'Feed Settings for ' + type(task).__name__,
             "feedType": ",".join(task.mapping_entities),
             "expectedDelivery": expected_delivery,
@@ -750,14 +794,14 @@ class Automation:
             logger.error('No data returned for this API call. %s', response.request.__dict__)
             return response_status, pandas.DataFrame()
 
-        df = pandas.read_json(response.text)
+        df = pandas.read_json(StringIO(response.text), typ='Series').to_frame().T
         df.columns = _column_name_cap(df.columns)
 
         return df
 
     @staticmethod
-    def deploy_on_timer(task, api_inputs: ApiInputs, cron_schedule, queue_name: QUEUE_NAME = "task",
-                        expected_delivery: EXPECTED_DELIVERY = "15min", settings: dict = None):
+    def deploy_on_timer(task, api_inputs: ApiInputs, cron_schedule, expected_delivery: EXPECTED_DELIVERY,
+                        queue_name: QUEUE_NAME = "task", settings: dict = None):
         """Deploy driver to run on timer.
 
         Parameters
@@ -768,18 +812,18 @@ class Automation:
             Object returned by initialize.initialize() function
         cron_schedule :
             Cron object containing the required schedule for the driver to be run.
-        queue_name : QUEUE_NAME, default = 'task'
-            The queue name.
         expected_delivery : EXPECTED_DELIVERY
-            The expected delivery frequency (Default value = "15min").
+            The expected delivery frequency.
+        queue_name : QUEUE_NAME, optional
+            The queue name (Default value = 'task').
         settings : dict, Optional
                 List of settings used to deploy the driver. For example, may contain the user_name and password
                 required to authenticate calls to a third-party API (Default value = None).
 
         Returns
         -------
-        str
-            A string containing the response text.
+        pandas.Dataframe
+            A dataframe containing the details of the deployed data feed.
 
         """
 
@@ -787,6 +831,16 @@ class Automation:
             logger.error("You must call initialize() before using API.")
             return pandas.DataFrame()
 
+        if not set([expected_delivery]).issubset(set(EXPECTED_DELIVERY.__args__)):
+            logger.error('expected_delivery parameter must be set to one of the allowed values defined by the '
+                         'EXPECTED_DELIVERY literal: %s', EXPECTED_DELIVERY.__args__)
+            return pandas.DataFrame()
+
+        if not set([queue_name]).issubset(set(QUEUE_NAME.__args__)):
+            logger.error('queue_name parameter must be set to one of the allowed values defined by the '
+                         'QUEUE_NAME literal: %s', QUEUE_NAME.__args__)
+            return pandas.DataFrame()
+
         if len(cron_schedule.split(' ')) != 7:
             logger.error("cron_schedule parameter must be in the format * * * * * *")
             return pandas.DataFrame()
@@ -814,7 +868,7 @@ class Automation:
         }
 
         payload = {
-            "driverId": task.id,
+            "driverId": str(task.id),
             "name": 'Timer Settings for ' + type(task).__name__,
             "feedType": ",".join(task.mapping_entities),
             "expectedDelivery": expected_delivery,
@@ -838,12 +892,13 @@ class Automation:
             logger.error('No data returned for this API call. %s', response.request.__dict__)
             return response_status, pandas.DataFrame()
 
-        df = pandas.read_json(response.text, typ='Series')
+        df = pandas.read_json(response.text, typ='Series').to_frame().T
+        df.columns = _column_name_cap(df.columns)
 
         return df
 
     @staticmethod
-    def cancel_deploy(task, api_inputs: ApiInputs, deploy_type: DEPLOY_TYPE):
+    def cancel_deploy(task, api_inputs: ApiInputs, deploy_type: DEPLOY_TYPE = 'timer'):
         """Deploy driver to run on timer.
 
         Parameters
@@ -852,8 +907,8 @@ class Automation:
             The custom task class created from the Abstract Base Class `Task`.
         api_inputs : ApiInputs
             Object returned by initialize.initialize() function
-        deploy_type : DEPLOY_TYPE
-            One of the deploy types defined by this lteral
+        deploy_type : DEPLOY_TYPE, optional
+            One of the deploy types defined by the DEPLOY_TYPE literal (Default value = 'timer')
 
 
         Returns
@@ -867,6 +922,15 @@ class Automation:
             logger.error("You must call initialize() before using API.")
             return pandas.DataFrame()
 
+        if str(deploy_type) != 'timer':
+            logger.error('The cancel_deploy() function currently only works for data feeds deployed on timer. ')
+            return
+
+        if not set([deploy_type]).issubset(set(DEPLOY_TYPE.__args__)):
+            logger.error('deploy_type parameter must be set to one of the allowed values defined by the '
+                         'DEPLOY_TYPE literal: %s', DEPLOY_TYPE.__args__)
+            return pandas.DataFrame()
+
         logger.info('Cancel deploy %s (%s) on timer for ApiProjectID: %s', type(task).__name__,
                     str(task.id), api_inputs.api_project_id)
         logger.info('Feed Type is %s', str(task.mapping_entities))
@@ -880,7 +944,7 @@ class Automation:
         # payload = {}
 
         url = api_prefix + api_inputs.datacentre + "/" + api_inputs.api_project_id + "/Automation/CancelTask/" + \
-              deploy_type + "/" + task.id
+              deploy_type + "/" + str(task.id)
 
         response = requests.get(url, headers=headers)
         response_status = '{} {}'.format(response.status_code, response.reason)
@@ -892,7 +956,8 @@ class Automation:
             logger.error('No data returned for this API call. %s', response.request.__dict__)
             return response_status, pandas.DataFrame()
 
-        df = pandas.read_json(response.text, typ='Series')
+        df = pandas.read_json(response.text, typ='Series').to_frame().T
+        df.columns = _column_name_cap(df.columns)
 
         return df
 
@@ -975,7 +1040,8 @@ class Automation:
         }
 
         url = api_prefix + api_inputs.datacentre + "/" + api_inputs.api_project_id + \
-              "/Automation/ListDataFeedsHistory/" + data_feed_id + "/" + str(top_count)
+              "/Automation/ListDataFeedsHistory/" + str(data_feed_id) + "/" + str(top_count)
+
         response = requests.request("GET", url, timeout=20, headers=headers)
 
         response_status = '{} {}'.format(response.status_code, response.reason)
@@ -1052,7 +1118,7 @@ class Automation:
         }
 
         url = api_prefix + api_inputs.datacentre + "/" + api_inputs.api_project_id + \
-              "/Automation/DataFeedsProcessOutput/" + data_feed_id + "/" + data_feed_file_status_id + "/" + \
+              "/Automation/DataFeedsProcessOutput/" + str(data_feed_id) + "/" + str(data_feed_file_status_id) + "/" + \
               str(row_number)
 
         response = requests.request("GET", url, timeout=20, headers=headers)
@@ -1077,6 +1143,116 @@ class Automation:
 
         return data_feed_file_process_output
 
+    @staticmethod
+    def data_feed_history_process_errors(api_inputs: ApiInputs, data_feed_id: uuid.UUID,
+                                         data_feed_file_status_id: uuid.UUID):
+        """Retrieve the unique error types for a given data feed file.
+
+        Retrieves the distinct error types present for the given `data_feed_id` and `data_feed_file_status_id`.
+
+        Parameters
+        ----------
+        api_inputs : ApiInputs
+            Object returned by initialize.initialize() function
+        data_feed_id : uuid.UUID
+            The unique identifier for the data feed that errors should be retrieved for.
+        data_feed_file_status_id : uuid.UUID
+            The unique identifier for the file processed for a given data feed that errors should be retrieved for.
+
+        Returns
+        -------
+        df : pandas.DataFrame
+            Dataframe containing the distinct error types present for the given data_feed_file_status_id.
+
+        """
+
+        if data_feed_id == '' or data_feed_file_status_id == '':
+            logger.error('The data_feed_id and data_feed_file_status_id parameters must be provided. ')
+            return
+
+        headers = {
+            'x-functions-key': api_inputs.api_key,
+            'Content-Type': 'application/json; charset=utf-8',
+            'user-key': api_inputs.user_id
+        }
+
+        url = api_prefix + api_inputs.datacentre + "/" + api_inputs.api_project_id + \
+              "/Automation/DataFeedsProcessErrors/" + str(data_feed_id) + "/" + str(data_feed_file_status_id)
+
+        response = requests.request("GET", url, timeout=20, headers=headers)
+
+        response_status = '{} {}'.format(response.status_code, response.reason)
+        if response.status_code != 200:
+            logger.error("API Call was not successful. Response Status: %s. Reason: %s.", response.status_code,
+                         response.reason)
+            return response_status, pandas.DataFrame()
+        elif len(response.text) == 0:
+            logger.error('No data returned for this API call. %s', response.request.__dict__)
+            return response_status, pandas.DataFrame()
+
+        df = pandas.read_json(response.text)
+        df.columns = _column_name_cap(df.columns)
+
+        return df
+
+    @staticmethod
+    def data_feed_history_errors_by_type(api_inputs: ApiInputs, data_feed_id: uuid.UUID,
+                                         data_feed_file_status_id: uuid.UUID, error_type: ERROR_TYPE):
+        """Retrieve the encountered errors for a given error type, data feed & file.
+
+        Retrieves the errors identified for a given `error_type` for the `data_feed_id` and `data_feed_file_status_id`.
+
+        Parameters
+        ----------
+        api_inputs : ApiInputs
+            Object returned by initialize.initialize() function
+        data_feed_id : uuid.UUID
+            The unique identifier for the data feed that errors should be retrieved for.
+        data_feed_file_status_id
+            The unique identifier for the data feed that errors should be retrieved for.
+        error_type: ERROR_TYPE
+            The error type of the structured logs to be retrieved.
+
+        Returns
+        -------
+        df : pandas.DataFrame
+            Dataframe containing the errors identified for the given `data_feed_file_status_id`.
+
+        """
+
+        if data_feed_id == '' or data_feed_file_status_id == '' or data_feed_id is None or data_feed_file_status_id is None:
+            logger.error('The data_feed_id and data_feed_file_status_id parameters must be provided. ')
+            return
+
+        if not set([error_type]).issubset(set(ERROR_TYPE.__args__)):
+            logger.error('error_type parameter must be set to one of the allowed values defined by the '
+                         'ERROR_TYPE literal: %s', ERROR_TYPE.__args__)
+            return pandas.DataFrame()
+
+        headers = {
+            'x-functions-key': api_inputs.api_key,
+            'Content-Type': 'application/json; charset=utf-8',
+            'user-key': api_inputs.user_id
+        }
+
+        url = api_prefix + api_inputs.datacentre + "/" + api_inputs.api_project_id + \
+              "/Automation/DataFeedsProcessGetErrorsByType/" + str(data_feed_id) + "/" + str(data_feed_file_status_id) \
+              + "/" + str(error_type)
+
+        response = requests.request("GET", url, timeout=20, headers=headers)
+
+        response_status = '{} {}'.format(response.status_code, response.reason)
+        if response.status_code != 200:
+            logger.error("API Call was not successful. Response Status: %s. Reason: %s.", response.status_code,
+                         response.reason)
+            return response_status, pandas.DataFrame()
+        elif len(response.text) == 0:
+            logger.error('No data returned for this API call. %s', response.request.__dict__)
+            return response_status, pandas.DataFrame()
+
+        df = pandas.read_csv(StringIO(response.text), sep=",", error_bad_lines=False)
+        return df
+
     @staticmethod
     def _get_task_code(task):
         """
